const { Sequelize } = require('sequelize')
const db = new Map()
module.exports = {
  port: 3002,
  static: {
    dir: 'public',
  },
  task: {
    dir: 'task',
  },
  xmlBody: {
    limit: 1024 * 100,
  },
  koaBody: {
    text: false,
    onError: (err, ctx) => {
      console.log('错误', err.message)
      throw new Error(err.message)
    },
  },
  response: {
    SUCCESS: (ctx, message) => {
      ctx.body = {
        code: 200,
        data: message,
        message: '操作成功',
      }
    },
    ERROR: (ctx, message) => {
      ctx.body = {
        code: 500,
        data: message,
        message: '发生错误',
      }
    },
    AUTH: (ctx, message) => {
      ctx.body = {
        code: 401,
        data: message,
        message: '你需要重新登录',
      }
    },
  },
  rateLimit: {
    driver: 'memory',
    db: db,
    duration: 500,
    errorMessage: 'Sometimes You Just Have to Slow Down.',
    id: (ctx) => ctx.ip,
    headers: {
      remaining: 'Rate-Limit-Remaining',
      reset: 'Rate-Limit-Reset',
      total: 'Rate-Limit-Total',
    },
    max: 1,
    disableHeader: false,
    whitelist: (ctx) => {
      return ['common/getLngAndLat', 'weixin/mp_login'].every((item) => {
        return !ctx.request.url.includes(item)
      })
      // some logic that returns a boolean
    },
    blacklist: (ctx) => {
      // some logic that returns a boolean
    },
  },
  db: {
    username: 'root',
    password: '123456789abc',
    host: '59acdf57310d9.sh.cdb.myqcloud.com',
    dialect: 'mysql',
    logging: false,
    timezone: '+08:00',
    port: 3306,
    pool: {
      max: 30,
      min: 1,
    },
    operatorsAliases: {
      $ne: Sequelize.Op.ne,
      $neq: Sequelize.Op.ne,
      $eq: Sequelize.Op.eq,
      $like: Sequelize.Op.like,
      $or: Sequelize.Op.or,
      $and: Sequelize.Op.and,
      $gt: Sequelize.Op.gt,
      $gte: Sequelize.Op.gte,
      $lt: Sequelize.Op.lt,
      $lte: Sequelize.Op.lte,
      $between: Sequelize.Op.between,
      $notIn: Sequelize.Op.notIn,
    },
  },
  app: {
    jwt: {
      secret: 'token',
      tokenKey: 'token',
      expiresIn: '60480000',
      whiteList: [], // 应用覆盖
      _whiteList: [
        '/',
        'log',
        'admin',
        'notify',
        'login',
        'logout',
        'find',
        'administrator/login',
        'administrator/checkLogin',
        'user/checkLogin',
        'user/login',
        'user/register',
        'common',
        'good/search',
        'category/tree',
        'system/initData',
      ], // 通用，会合并
    },
  },
  restc: {
    excludes: ['admin'],
  },
  cacheModel: [],
}
