const { getAppByCtx, lodash } = require('multiple-quick-koa')

exports.bulkUpdate = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  const { list, type = 'key' } = ctx.request.body
  if (type === 'key') {
    await app.model.routes.bulkCreate(
      list.map((item) => {
        return {
          id: item.id,
          key: `${item.model.belongs}/${item.model.model}`,
        }
      }),
      {
        updateOnDuplicate: ['key'],
      }
    )
  } else if (type === 'model') {
    const modelList = await app.model.model.findAll()

    await app.model.routes.bulkCreate(
      list
        .filter((item) => {
          const target = modelList.find(
            (m) => m.model === item.key.split('/')[1]
          )
          return target
        })
        .map((item) => {
          const target = modelList.find(
            (m) => m.model === item.key.split('/')[1]
          )
          return {
            id: item.id,
            model_id: target.id,
          }
        }),
      {
        updateOnDuplicate: ['model_id'],
      }
    )
  }

  ctx.SUCCESS('ok')
}

exports.init = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  const { list } = ctx.request.body

  const result = await app.model.routes.bulkCreate(list)

  ctx.SUCCESS(result)
}

exports.initRoute = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  if (!app.model.routes) return Promise.resolve()
  const routesList = await app.model.routes.findAll()
  await app.model.routes.sync({
    force: true,
  })

  let list = [
    {
      id: 1,
      name: '首页',
      icon: 's-home',
      parent_id: 0,
      url: '/dashboard',
      model_id: 0,
    },
    {
      id: 2,
      name: '项目管理',
      icon: 's-operation',
      parent_id: 0,
      url: '',
      model_id: 0,
    },
    {
      id: 3,
      name: '系统管理',
      icon: 's-operation',
      parent_id: 0,
      url: '',
      model_id: 0,
    },
    {
      id: 4,
      name: '管理设置',
      icon: 's-operation',
      parent_id: 0,
      url: '',
      model_id: 0,
    },
  ]

  const permissionList = await app.model.permission.findAll()
  const modelList = await app.model.model.findAll()
  const roleList = await app.model.role.findAll()
  const rolePermission = lodash.uniq(
    roleList
      .map((item) => item.toJSON())
      .reduce((a, b) => [...a, ...b.permission_id], [])
  )
  for (let i = 0; i < permissionList.length; i++) {
    const target = permissionList[i].toJSON()
    if (target.controller === '') {
      await app.model.permission.destroy({
        where: {
          id: target.id,
        },
      })
      continue
    }
    const model = modelList.find((item) => item.model === target.controller)
    if (!rolePermission.includes(target.id)) {
      continue
    }

    let parent_id = 2
    if (model && model.belongs === 'setting') {
      parent_id = 3
    } else if (model && model.belongs === 'auth') {
      parent_id = 4
    }
    const is_project = !model
    list = [
      ...list,
      {
        name: target.name,
        parent_id,
        url: is_project ? `/project/${appName}/` + target.controller : '',
        model_id: model ? model.id : 0,
        key: is_project
          ? ''
          : model && model.belongs
          ? `${model.belongs}/${model.model}`
          : '',
      },
    ]
  }
  const sortList = list.map((item) => {
    const sort = routesList.find((r) => r.name === item.name)
    return {
      ...item,
      sortOrder: item.id ? item.id : sort ? sort.sortOrder : list.length,
    }
  })
  await app.model.routes.bulkCreate(sortList)
  ctx.SUCCESS('ok')
}
