const { Sequelize, Random, lodash, moment } = require('multiple-quick-koa')

exports.name = {
  type: Sequelize.STRING(),
  comment: '分类名',
  allowNull: false,
  is_mock: false,
  defaultValue: '',
  sortOrder: 1,
}

exports.code = {
  type: Sequelize.STRING(),
  comment: 'key值',
  allowNull: false,
  is_mock: false,
  defaultValue: '',
  sortOrder: 2,
}

exports.type = {
  type: Sequelize.ENUM(
    'switch',
    'input',
    'image',
    'decimal',
    'file',
    'textarea',
    'rich-text',
    'date',
    'datetime',
    'list'
  ),
  comment: 'type类型',
  allowNull: true,
  is_mock: false,
  defaultValue: 'input',
  sortOrder: 3,
}

exports.value = {
  type: Sequelize.TEXT('long'),
  comment: 'value值',
  allowNull: true,
  is_mock: false,
  defaultValue: null,
  sortOrder: 4,
  get: function () {
    const type = this.getDataValue('type')
    switch (type) {
      case 'switch':
        return Boolean(Number(this.getDataValue('value')))
      case 'list':
        return JSON.parse(this.getDataValue('value'))
      case 'input':
        return this.getDataValue('value')
      case 'decimal':
        return lodash.isNaN(Number(this.getDataValue('value')))
          ? 0
          : Number(this.getDataValue('value'))
      case 'datetime':
        return moment(this.getDataValue('value')).format('YYYY-MM-DD HH:mm:ss')
      case 'date':
        return moment(this.getDataValue('value')).format('YYYY-MM-DD')
      case 'image':
        return this.getDataValue('value').startsWith('//')
          ? `https:${this.getDataValue('value')}`
          : this.getDataValue('value')
      default:
        return this.getDataValue('value')
    }
  },
}

exports.parent_id = {
  type: Sequelize.TINYINT.UNSIGNED,
  comment: '上级分类',
  allowNull: false,
  is_mock: false,
  defaultValue: 0,
  sortOrder: 5,
}

exports.is_front = {
  type: Sequelize.BOOLEAN,
  comment: '是否前端可用',
  allowNull: false,
  is_mock: false,
  defaultValue: true,
  sortOrder: 6,
  extra: {
    controlInline: true,
  },
}

exports.is_cache = {
  type: Sequelize.BOOLEAN,
  comment: '是否后端缓存',
  allowNull: false,
  is_mock: false,
  defaultValue: true,
  sortOrder: 7,
  extra: {
    controlInline: true,
  },
}

exports.is_control = {
  type: Sequelize.BOOLEAN,
  comment: '是否可控',
  allowNull: false,
  is_mock: false,
  defaultValue: true,
  sortOrder: 8,
  extra: {
    controlInline: true,
  },
}
