const fs = require('fs')
const path = require('path')
const { VM } = require('vm2')
const axios = require('axios')
const cheerio = require('cheerio')
const {
  lodash,
  getAppByCtx,
  getConfig,
  Sequelize,
} = require('multiple-quick-koa')

exports.initModel = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  const { model, option } = ctx.request.body
  const config = option
    ? {
        [option]: true,
      }
    : {
        alter: true,
      }
  if (model === 'all') {
    const modelList = Object.keys(app.model)
    await Promise.all(modelList.map((m) => app.model[m].sync(config)))
    return ctx.SUCCESS('ok')
  }
  if (app.model[model]) {
    if (Object.keys(app.model[model]).length === 0)
      return ctx.ERROR('请检查，model为空')
    const result = await app.model[model].sync(config)
    return ctx.SUCCESS(result)
  }
  return ctx.SUCCESS(null)
}

exports.customRouter = async (ctx) => {
  // const {
  //   app,
  //   appName,
  // } = getAppByCtx(ctx);

  // const list = [{
  //   path: 'jiangong_statis',
  //   name: '数据面板'
  // }]
  const list = []
  ctx.SUCCESS(list)
  return list
}

exports.showTables = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  if (app.cache && app.cache.get('showTables')) {
    ctx.SUCCESS(app.cache.get('showTables'))
    return app.cache.get('showTables')
  }

  const pluginDir = path.resolve(
    __dirname,
    `${process.cwd()}/app/${appName}/plugins`
  )
  let aliasModelList = []
  fs.readdirSync(pluginDir).forEach((folder) => {
    const isFolder = fs.lstatSync(path.resolve(pluginDir, folder)).isDirectory()
    const hasConfig = fs.existsSync(
      path.resolve(
        __dirname,
        `${process.cwd()}/app/${appName}/plugins/${folder}/config.js`
      )
    )
    let hasModel = false
    if (hasConfig) {
      const folderConfig = require(`${process.cwd()}/app/${appName}/plugins/${folder}/config.js`)
      if (folderConfig.model) {
        hasModel = true
      }
    }

    if (
      isFolder &&
      hasConfig &&
      hasModel &&
      !Object.keys(app.model).includes(folder)
    ) {
      aliasModelList = [...aliasModelList, folder]
    }
  })
  const modelList = [...Object.keys(app.model), ...aliasModelList]
  const models = modelList.map((_model) => {
    const defaultExcludes = [
      'id',
      'created_at',
      'updated_at',
      'deleted_at',
      'createdid',
    ]
    const mock = !lodash.isEmpty(
      app.mock && app.mock[_model] && app.mock[_model]()
    )
    const defaultConfigPath = path.resolve(__dirname, `../${_model}/config.js`)
    const defaultConfigExist = fs.existsSync(defaultConfigPath)
    const target =
      Object.keys(app.config[_model]).length === 0 && defaultConfigExist
        ? require(defaultConfigPath)
        : app.config[_model]
    // const target = Object.keys(app.config[model])

    // console.log(model,mock,app.mock && app.mock[model] && app.mock[model]())
    const model = target.model ? target.model : _model
    return {
      modelName: _model,
      model,
      name: target ? target.name : '',
      // 属于哪个分类
      belongs: lodash.get(target, 'belongs', 'page'),
      // list切换tab显示 ex lanuage
      limit: lodash.get(target, 'limit', 20),
      excludeAuth: lodash.get(target, 'excludeAuth', []),
      // 可排序字段
      order: lodash.get(target, 'order', []),
      // 默认排序
      defaultOrder: lodash.get(target, 'defaultOrder', []),
      // 下拉筛选
      select: lodash.get(target, 'select', []),
      // 是否可以选择批量删除
      multiple: lodash.get(target, 'multiple', true),
      // 是否后台拆分sql
      is_split: lodash.get(target, 'is_split', false),
      // 是否后台拆分count
      is_split_count: lodash.get(target, 'is_split_count', false),
      // 是否后台显示虚拟字段
      show_virtual: lodash.get(target, 'show_virtual', false),
      // 忽略字段
      excludes:
        _model === 'setting'
          ? lodash.get(target, 'excludes', [])
          : lodash.difference(
              lodash.uniq([
                ...defaultExcludes,
                ...lodash.get(target, 'excludes', []),
              ]),
              lodash.get(target, 'order', [])
            ),
      // reference admin components
      reference: lodash.get(target, 'reference', []),
      // reference admin components
      personal: lodash.get(target, 'personal', []),
      // sortOrder router
      sortOrder: lodash.get(target, 'sortOrder', 1),

      attributes: lodash.mapValues(
        app.attributes[model],
        (item) => item.comment
      ),

      // include for admin
      include: lodash.get(target, 'include', ''),
      // only include exsit， and for custom like user_distribute has two user model
      relate: lodash.get(target, 'relate', ''),
      // can be mock boolean
      mock,
      // reference model query
      referenceSelect: lodash.get(target, 'referenceSelect', []),
      // 是否可以选择批量删除
      availableSort: lodash.get(target, 'availableSort', false),
      // 用于后台管理不要查询太多数据，常用于表多的model
      autoData: lodash.get(target, 'autoData', null),
      // 有些model需要前置查询其它model数据
      initList: lodash.get(target, 'initList', []),
      // 局部更新列表
      editInline: lodash.get(target, 'editInline', {}),
      // 有些model需要前置查询其它model数据
      comment: lodash.get(target, 'comment', {}),
      // 默认查询，覆盖之前
      modelQuery: lodash.get(target, 'modelQuery', {}),
      // 删除时检查是不是含有
      deleteCheckList: lodash.get(target, 'deleteCheckList', []),
    }
  })

  ctx.SUCCESS(models)
  app.cache && app.cache.set('showTables', models)
  return models
}

exports.getTable = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { model: _model, show_virtual = false } = ctx.request.body

  if (app.cache && app.cache.get(`${_model}-table`)) {
    return ctx.SUCCESS(app.cache.get(`${_model}-table`))
  }

  const model = app.config[_model].model || _model

  const _result = await app.sequelize.getQueryInterface().describeTable(model)

  const result = lodash.pick(_result, [
    ...Object.keys(app.attributes[model]),
    'id',
    'createdid',
    'updated_at',
    'created_at',
  ])

  const obj = {}
  if (show_virtual) {
    Array.from(
      new Set([...Object.keys(result), ...Object.keys(app.attributes[model])])
    ).forEach((attr) => {
      obj[attr] = lodash.merge(
        app.attributes[model][attr],
        result[attr],
        app.config[_model].comment && app.config[_model].comment
          ? app.config[_model].comment[attr]
          : {}
      )
    })
  } else {
    Object.keys(result).forEach((attr) => {
      obj[attr] = lodash.merge(
        app.attributes[model][attr],
        result[attr],
        app.config[_model].comment && app.config[_model].comment
          ? app.config[_model].comment[attr]
          : {}
      )
    })
  }

  app.cache && app.cache.set(`${_model}-table`, obj)
  return ctx.SUCCESS(obj)
}

exports.dropModel = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { model } = ctx.request.body
  const result = await app.model[model].drop()
  return ctx.SUCCESS(result)
}

exports.getImage = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const appConfig = getConfig(app)

  const { site_host } = await appConfig.getObject('base')
  // const { dir, upload } = app.config.static;
  const targetDir = `${path.resolve(
    __dirname,
    `${process.cwd()}/public/upload`
  )}`
  const files = fs.readdirSync(targetDir)
  const images = files.filter((file) =>
    /\w(\.gif|\.jpeg|\.png|\.jpg|\.bmp)/i.test(file)
  )
  const result = []
  for (let i = 0; i < images.length; i++) {
    const image = `${path.resolve(
      __dirname,
      `${process.cwd()}/public/upload/${images[i]}`
    )}`

    result.push({
      img: `http://${site_host || 'api.kuashou.com'}/upload/${images[i]}`,
      name: images[i],
      ...fs.statSync(image),
    })
  }

  ctx.SUCCESS(result)
}

exports.mock = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { model } = ctx.request.body
  const result = app.mock && app.mock[model] && app.mock[model]()
  return ctx.SUCCESS(!lodash.isEmpty(result))
}

exports.table = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  const { model } = ctx.request.body
  if (!model) {
    const result = Object.keys(app.model).map((m) => ({
      [m]: lodash.mapValues(app.attributes[m], (item) => item.comment),
    }))
    ctx.SUCCESS(result)
  } else {
    const result = app.attributes[model]
    ctx.SUCCESS(result)
  }
}

exports.showAllSchemas = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { model } = ctx.request.body
  const result = await app.sequelize.getQueryInterface().describeTable(model)
  return ctx.SUCCESS(result)
}

exports.initDataWithCache = exports.initData = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  const { includes, excludes = [] } = ctx.request.body
  const result = await app.service.system.initData({
    app,
    includes,
    excludes,
    ctx,
  })

  return ctx.SUCCESS(result)
}

exports.recentlyError = async (ctx) => {
  // const {
  //   app,
  //   appName,
  // } = getAppByCtx(ctx);

  if (ctx.ws) {
    const ws = await ctx.ws()
    const send = (data) => ws.send(JSON.stringify(data))
    ctx.app.on('error', async (err) => {
      console.log(err)
      try {
        send({
          type: 'error',
          payload: err ? err.errorData : {},
        })
      } catch (e) {
        console.log(e)
      }
    })
  } else {
    throw new Error('这是一个websocket url')
  }
}

exports.sandbox = async (ctx) => {
  const code = `
          module.exports = async (callback)=>{
              callback({
                name:1
              })
          }
      `
  // const { code } = ctx.request.body;
  const vm = new VM({
    sandbox: {
      axios,
      cheerio,
      module,
      setTimeout,
      sleep: (time) =>
        new Promise((resolve) => setTimeout(resolve, time * 1000)),
    },
  })
  vm.run(code)(ctx.SUCCESS)
}

exports.onlineChat = async (ctx) => {
  if (ctx.ws) {
    const ws = await ctx.ws()
    const send = (data) => ws.send(JSON.stringify(data))
    ws.on('message', async (code) => {
      const vm = new VM({
        sandbox: {
          send: (data) => ws.send(JSON.stringify(data)),
          axios,
          cheerio,
        },
      })
      vm.run(code)
      const unhandledRejections = {}

      process.on('unhandledRejection', (reason) => {
        if (unhandledRejections[reason]) {
          console.log('====?', reason)
        } else {
          send({
            type: 'error',
            payload: `${reason}`,
          })
          unhandledRejections[reason] = true
        }
      })
    })
  } else {
    throw new Error('这是一个websocket url')
  }
}

exports.chat = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  if (ctx.ws) {
    const ws = await ctx.ws()
    const send = (data) => ws.send(JSON.stringify(data))
    ws.on('message', async (payload) => {
      const message = JSON.parse(payload)
      const result = await app.model.ask.create({
        user_id: message.payload.user_id,
        content: message.payload.content,
        is_ask: Boolean(message.payload.user_id),
      })

      send({
        type: 'message',
        payload: result,
      })
    })
  } else {
    throw new Error('这是一个websocket url')
  }
}

exports.task = async (ctx) => {
  const { app } = getAppByCtx(ctx)

  const { task } = ctx.request.body

  if (app.task && app.task[task]) {
    await app.task[task](app)
  }
  ctx.SUCCESS(task)
}

exports.dashboard = async (ctx) => {
  // const {
  //   app,
  // } = getAppByCtx(ctx);

  const result = [
    //   {
    //   name: '用户',
    //   number: 5,
    //   route: 'page/user'
    // }
  ]
  ctx.SUCCESS(result)
}

exports.drop = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  const result = await app.sequelize.showAllSchemas()
  const list = result.map((item) => item[`Tables_in_${appName}`])
  const filterList = list.filter((item) => !app.model[item])
  await Promise.all(
    filterList.map((folder) => {
      const model = app.sequelize.define(
        folder,
        {
          id: {
            type: Sequelize.INTEGER(11),
            allowNull: false,
            primaryKey: true,
            autoIncrement: true,
          },
        },
        {
          freezeTableName: true,
          comment: folder,
          createdAt: 'created_at',
          updatedAt: 'updated_at',
          name: {
            singular: folder,
            plural: `${folder}s`,
          },
        }
      )
      return model.drop()
    })
  )

  ctx.SUCCESS(`删除${filterList.length}个无用表`)
}

exports.mockPay = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  const { order_id, order_price, prefix, ...rest } = ctx.request.body
  const model = prefix ? `${prefix}_order` : 'order'
  console.log(model, 'model')
  if (!(app.service[model] && app.service[model].notify)) {
    throw new Error(`没配置${model}支付回调notify`)
  }
  await app.service[model].notify({
    app,
    order_id,
    order: model,
    order_price,
    ...rest,
  })
  ctx.SUCCESS('ok')
}
