const { getConfig, lodash } = require('multiple-quick-koa')
const formatPostFunction = (str) => `eval(${str})`
const nodeVm = require('vm')

exports.initData = async ({ includes, excludes, app, ctx }) => {
  const appConfig = getConfig(app)
  const { version: cacheVersion } = await appConfig.getObject('base')
  const apiVersion = ctx.request.header.version

  if (!(excludes instanceof Array)) {
    excludes = [excludes]
  }
  const result = await Promise.all(
    Object.keys(includes)
      .filter((item) => excludes.includes(item) || excludes.length === 0)
      .map((item) => {
        const route = includes[item]
        if (lodash.isObject(route)) {
          const {
            url,
            data,
            order,
            limit,
            include,
            excludeInclude,
            autoInclude = true,
          } = route
          const [model, fn] = url.split('/')

          let myInclude

          if (include) {
            const _str = formatPostFunction(include)
            const postInclude = nodeVm.runInNewContext(_str)
            // const postInclude = include.toFunction();
            myInclude =
              app.include && app.include[model]
                ? [
                    ...postInclude(app),
                    ...(autoInclude ? app.include[model] : []),
                  ]
                : postInclude(app)
          } else {
            myInclude = autoInclude ? app.include[model] : []
          }

          if (
            excludeInclude &&
            Array.isArray(excludeInclude) &&
            excludeInclude.length > 0
          ) {
            myInclude = myInclude.filter((i) => {
              if (excludeInclude.includes(model)) return false
              if (excludeInclude.includes(i.as)) return false
              return (
                (excludeInclude.every((m) => {
                  return app.model[m] !== i.model
                }) ||
                  i.as) &&
                !(
                  i.as &&
                  data.attributes &&
                  data.attributes.exclude &&
                  data.attributes.exclude.includes(i.as + '_id')
                )
              )
            })
          }

          if (app.controller[model] && app.controller[model][fn]) {
            return app.controller[model][fn](ctx, data).then((result) => {
              if (!result) {
                console.log(`需要在控制器${fn}中返回`)
              }
              return {
                [item]: result,
              }
            })
          } else if (app.model[model]) {
            return app.model[model][fn]
              ? app.model[model][fn]({
                  order,
                  limit,
                  include: myInclude,
                  ...data,
                }).then((result) => {
                  const list =
                    data && data.omit && Array.isArray(data.omit)
                      ? result.map((obj) => {
                          return lodash.omit(obj.toJSON(), data.omit)
                        })
                      : result
                  return {
                    [item]: list,
                  }
                })
              : Promise.resolve({
                  [item]: [],
                })
          } else {
            return Promise.resolve({
              [item]: [],
            })
          }
        }
        const [model, fn] = route.split('/')
        if (app.cache && app.cache.get(`${model}`)) {
          return Promise.resolve({
            [item]: app.cache.get(`${model}`),
          })
        }
        return app.model[model][fn]({
          include: app.include[model],
          ...(app.config[model] ? app.config[model].autoData : {}),
        }).then((res) => {
          app.cache && app.cache.set(`${model}`, res)
          const cacheModel = lodash.get(app, 'appConfig.cacheModel', [])
          app.appConfig.cacheModel = Array.from(new Set([...cacheModel, model]))
          return {
            [item]: res,
          }
        })
      })
  )

  const obj = result.reduce(
    (a, b) => ({
      ...a,
      ...b,
    }),
    {}
  )
  if (obj.setting && apiVersion && cacheVersion) {
    return {
      ...obj,
      setting: obj.setting.map((i) => {
        if (i.code === 'is_check' && i.value === false) {
          return {
            ...i.toJSON(),
            value: apiVersion === cacheVersion,
          }
        }
        return i
      }),
    }
  }
  return obj
}
