const {
  getAppByCtx,
  getConfig,
  getService,
  lodash,
} = require('multiple-quick-koa')

const util = require('util')
const axios = require('axios')
const qr = require('qr-image')
const path = require('path')
const TOUTIAO = getService('toutiao')

const utils = require('../../utils')

exports.initData = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  if (app.cache.get('initData')) {
    return ctx.SUCCESS(app.cache.get('initData'))
  } else {
    return ctx.SUCCESS(null)
  }
}
exports.qr_code = async (ctx) => {
  const { app } = getAppByCtx(ctx)
  const { page = 'pages/index/index', type = 'douyin', item } = ctx.request.body

  const appConfig = getConfig(app)
  // const { app_id, app_secrect } = await appConfig.getObject('toutiao')

  const { site_host } = await appConfig.getObject('base')

  const toutiaoService = new TOUTIAO({
    appid: item.appid,
    secrect: item.secrect,
    targetPath: `${path.resolve(__dirname, `${process.cwd()}/public/upload`)}`,
  })

  toutiaoService.init()

  const result = await toutiaoService.createQR({
    path: encodeURIComponent(page),
    appname: type,
    filename: utils.hashFn(item.key + page + type),
  })
  console.log(`qr,${item.key},${page},${type}`)
  const qr_image = `https://${site_host || 'api.kuashou.com'}/upload/${result}`

  // const qr_image = `http://localhost:3001/upload/${result}`;
  // await app.model.user.upsert({
  //   id: lodash.get(ctx.request, `${appName}-user.id`, null),
  //   qr_image
  // })

  ctx.SUCCESS(qr_image)
}

exports.getCode = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  // let url = req.query.url
  // let img = qr.image(url, { size: 10 })
  // res.writeHead(200, {
  //     'Content-Type': 'image/png'
  // })
  // img.pipe(res)

  const appConfig = getConfig(app)
  const { app_id, app_secrect } = await appConfig.getObject('toutiao')

  const url =
    'https://open.douyin.com/platform/oauth/connect?client_key=awbzmjjjcusg3hg1&redirect_uri=http://wxdy.592web.com/douyin/auth_redirect&response_type=code&scope=aweme.share,user_info,fans.data,renew_refresh_token,micapp.is_legal,following.list,video.comment,im&state=1613f649-7e6c-4d86-945b-eea62ea00b09'

  // const result = await axios.get(url, {
  //   client_key: app_id,
  // }).then(res => res.data)

  const img = qr.image(url, {
    size: 10,
  })
  ctx.type = 'image/png'
  ctx.body = img

  // ctx.SUCCESS(result)
}

exports.login = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)

  const { code, anonymousCode, ...rest } = ctx.request.body

  const appConfig = getConfig(app)
  const { app_id, app_secrect } = await appConfig.getObject('toutiao')

  const loginUrl =
    'https://developer.toutiao.com/api/apps/jscode2session?appid=%s&secret=%s&code=%s&anonymous_code=%s'

  const url = util.format(loginUrl, app_id, app_secrect, code, anonymousCode)

  const res = await axios.get(url).then((res) => res.data)

  const { openid } = res

  if (!openid) {
    const msg = `找不到openid, errcode:${res.errcode} app_id:${app_id} app_secrect:${app_secrect}`
    console.log(msg)
    throw new Error('头条登录失败')
  }

  const post = lodash.pickBy(
    {
      nick_name: lodash.get(rest, 'nickName', null),
      avatar: lodash.get(rest, 'avatarUrl', null),
      country: lodash.get(rest, 'country', null),
      city: lodash.get(rest, 'city', null),
      province: lodash.get(rest, 'province', null),
      gender: lodash.get(rest, 'gender', null),
      openid,
      user_id: lodash.get(ctx.request, `${appName}-user.id`, null),
    },
    (item) => item !== null
  )

  const openIdUser = await app.model.toutiao_user.findOne({
    where: {
      openid: post.openid,
    },
  })

  let uid
  if (openIdUser) {
    await app.model.toutiao_user.update(post, {
      where: {
        openid: post.openid,
      },
    })
    uid = openIdUser.user_id
  } else {
    if (post.user_id) {
      uid = post.user_id
    } else {
      const user = await app.model.user.create()
      uid = user.id
    }

    await app.model.toutiao_user.create({
      ...post,
      user_id: uid,
    })
  }

  const result = await app.model.user.findOne({
    where: {
      id: uid,
    },
    include: [
      {
        model: app.model.h5_user,
      },
      {
        model: app.model.mp_user,
      },
      {
        model: app.model.toutiao_user,
      },
      ...app.include.user,
    ],
  })

  console.log('result', result, app.include.user)

  const tokenResult = {
    id: result.id,
    name: result.name,
    mobile: result.mobile,
    h5_user: result.h5_user
      ? {
          openid: result.h5_user && result.h5_user.openid,
        }
      : null,
    mp_user: result.mp_user
      ? {
          openid: result.mp_user && result.mp_user.openid,
        }
      : null,
    toutiao_user: result.toutiao_user
      ? {
          openid: result.toutiao_user && result.toutiao_user.openid,
        }
      : null,
  }

  const token = await app.sign({
    user: tokenResult,
  })

  return ctx.SUCCESS({
    token,
    user: result,
  })
}
