const md5 = require('js-md5')
const { getAppByCtx, getConfig } = require('multiple-quick-koa')

exports.login = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  const { mobile, password, ...rest } = ctx.request.body
  let result = await app.model.user.findOne({
    where: {
      mobile,
      password: [md5(password), password],
      ...rest,
    },
  })
  if (!result) throw new Error('账号密码错误')
  console.log('user==>', ctx.request[`${appName}-user`])
  if (
    ctx.request[`${appName}-user`] &&
    ctx.request[`${appName}-user`].mp_user
  ) {
    // 如果还有第三方账号app.model.xx_user，也有同样的分支逻辑

    /**
     * 绑定匿名用户，更新成当前用户
     */

    await app.model.mp_user.update(
      {
        user_id: result.id,
      },
      {
        where: {
          user_id: ctx.request[`${appName}-user`].id,
        },
      }
    )
  }

  if (
    ctx.request[`${appName}-user`] &&
    ctx.request[`${appName}-user`].h5_user
  ) {
    await app.model.h5_user.update(
      {
        user_id: result.id,
      },
      {
        where: {
          user_id: ctx.request[`${appName}-user`].id,
        },
      }
    )
  }

  if (
    ctx.request[`${appName}-user`] &&
    ctx.request[`${appName}-user`].github_user
  ) {
    await app.model.github_user.update(
      {
        user_id: result.id,
      },
      {
        where: {
          user_id: ctx.request[`${appName}-user`].id,
        },
      }
    )
  }
  /**
   * 查出最新用户，存token并返回
   */
  result = await app.model.user.findOne({
    where: {
      id: result.id,
    },
    include: [
      {
        model: app.model.github_user,
      },
      {
        model: app.model.h5_user,
      },
      {
        model: app.model.mp_user,
      },
      {
        model: app.model.toutiao_user,
      },
      ...app.include.user,
    ].filter((item) => item.model),
  })

  const tokenResult = {
    id: result.id,
    name: result.name,
    mobile: result.mobile,
    h5_user: result.h5_user
      ? {
          openid: result.h5_user && result.h5_user.openid,
        }
      : null,
    mp_user: result.mp_user
      ? {
          openid: result.mp_user && result.mp_user.openid,
        }
      : null,
    toutiao_user: result.toutiao_user
      ? {
          openid: result.toutiao_user && result.toutiao_user.openid,
        }
      : null,
  }
  const token = await app.sign({
    user: tokenResult,
  })

  return ctx.SUCCESS({
    token,
    user: result,
  })
}

exports.register = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  const { name, mobile, code, password } = ctx.request.body

  const result = await app.model.user.findOne({
    where: {
      mobile,
      code,
    },
  })

  if (!result) {
    throw new Error('手机号和验证码不一致')
  }

  if (
    ctx.request[`${appName}-user`] &&
    ctx.request[`${appName}-user`].mp_user
  ) {
    // 如果还有第三方账号app.model.xx_user，也有同样的分支逻辑

    /**
     * 绑定匿名用户，更新成当前用户
     */

    await app.model.mp_user.update(
      {
        user_id: result.id,
      },
      {
        where: {
          user_id: ctx.request[`${appName}-user`].id,
        },
      }
    )
  }

  await app.model.user.update(
    {
      name,
      password,
    },
    {
      where: {
        mobile,
      },
    }
  )

  const userResult = await app.model.user.findOne({
    where: {
      id: result.id,
    },
    include: [
      {
        model: app.model.h5_user,
      },
      {
        model: app.model.mp_user,
      },
      ...app.include.user,
    ].filter((item) => item.model),
  })

  const token = await app.sign({
    user: userResult,
  })

  return ctx.SUCCESS({
    token,
    user: result,
  })
}

exports.checkLogin = async (ctx) => {
  const { app, appName } = getAppByCtx(ctx)
  const { config: _config } = ctx.request.body
  const config = _config || 'weixin_mp'
  const appConfig = getConfig(app)
  const { app_id } = await appConfig.getObject(config)
  if (ctx.request[`${appName}-user`] && ctx.request[`${appName}-user`].id) {
    const h5_user_include = config.includes('h5')
      ? {
          model: app.model.h5_user,
          where: {
            appid: app_id,
          },
          attributes: {
            exclude: ['created_at', 'updated_at', 'createdid'],
          },
        }
      : {
          model: app.model.h5_user,
        }
    const mp_user_include = config.includes('mp')
      ? {
          model: app.model.mp_user,
          where: {
            appid: app_id,
          },
        }
      : {
          model: app.model.mp_user,
        }
    let result
    const includeDefault = [
      {
        model: app.model.github_user,
      },
      h5_user_include,
      mp_user_include,
      {
        model: app.model.toutiao_user,
      },
      ...app.include.user,
    ].filter((item) => item.model)
    if (app.appConfig.loginData) {
      const include = includeDefault.filter((i) => {
        return app.appConfig.loginData.excludeInclude.every((m) => {
          return app.model[m] !== i.model
        })
      })
      result = await app.model.user.findOne({
        where: {
          id: ctx.request[`${appName}-user`].id,
        },
        attributes: app.appConfig.loginData.attributes,
        include,
      })
    } else {
      result = await app.model.user.findOne({
        where: {
          id: ctx.request[`${appName}-user`].id,
        },
        include: includeDefault,
      })
    }

    if (!result) {
      return ctx.SUCCESS({
        token: null,
        user: null,
      })
    }

    if (config.includes('mp')) {
      const appConfig = getConfig(app)
      const { app_id } = await appConfig.getObject(config)
      if (
        !result.mp_user ||
        (result.mp_user.appid && result.mp_user.appid !== app_id)
      ) {
        return ctx.SUCCESS({
          token: null,
          user: null,
        })
      }
    } else if (config.includes('h5')) {
      const appConfig = getConfig(app)
      const { app_id } = await appConfig.getObject(config)
      if (
        !result.h5_user ||
        (result.h5_user.appid && result.h5_user.appid !== app_id)
      ) {
        return ctx.SUCCESS({
          token: null,
          user: null,
        })
      }
    }

    const tokenResult = {
      id: result.id,
      name: result.name,
      mobile: result.mobile,
      h5_user: result.h5_user
        ? {
            openid: result.h5_user && result.h5_user.openid,
          }
        : null,
      mp_user: result.mp_user
        ? {
            openid: result.mp_user && result.mp_user.openid,
          }
        : null,
      toutiao_user: result.toutiao_user
        ? {
            openid: result.toutiao_user && result.toutiao_user.openid,
          }
        : null,
    }
    const token = await app.sign({
      user: tokenResult,
    })

    return ctx.SUCCESS({
      token,
      user: result,
    })
  }

  return ctx.SUCCESS({
    token: null,
    user: null,
  })
}
exports.logout = async (ctx) => ctx.SUCCESS()
