const axios = require('axios');
const APP = require('multiple-quick-koa');
const APP_CONFIG = require('../../config');
const config = require('./config');

const modelName = __dirname.split('/').reverse()[0];
const service = axios.create({
  baseURL: `http://localhost:${APP_CONFIG.port}/${modelName}`,
});
service.interceptors.request.use((config) => { config.headers['Client-Type'] = 2; return config; });

service.interceptors.response.use(response => response.data, error => Promise.reject(error));

const sleep = time => new Promise(resolve => setTimeout(resolve, time * 1000));

describe(`${config.name}/${modelName}`, () => {
  beforeAll(async () => {
    try {
      await axios.post('/');
    } catch (e) {
      console.log('正在启动服务器');
      const app = new APP(APP_CONFIG);
      await app.start();
    }
  });

  test('错误登录', async () => {
    const res = await service.post('login', {
      name: 'chenlin1',
      password: 'sugar23',
    });
    expect(res.code).toBe(500);
  });

  test('正确登录', async () => {
    const res = await service.post('login', {
      name: 'chenlin',
      password: 'sugar23',
    });
    expect(res.code).toBe(200);
  });

  test('获取token', async () => {
    const { data: { token } } = await service.post('login', {
      name: 'chenlin',
      password: 'sugar23',
    });
    expect(Boolean(token)).toBe(true);
  });

  test('没有token信息', async () => {
    const res = await service.post('getInfo', {});
    expect(res.code).toBe(401);
  });

  test('获取用户信息', async () => {
    const { data: { token } } = await service.post('login', {
      name: 'chenlin',
      password: 'sugar23',
    });

    const res = await service.post('getInfo', {}, {
      headers: {
        'Client-Type': 2,
        'X-Token': token,
      },
    });

    expect(res.code).toBe(200);
  });
});
