const { getAppByCtx, getConfig } = require('multiple-quick-koa')
const { Pay } = require('@sigodenjs/wechatpay')
const fs = require('fs')
const path = require('path')

exports.refund = async ({
  ctx,
  id,
  prefix = '',
  total_fee,
  refund_fee,
  price,
  ...rest
}) => {
  const { app, appName } = getAppByCtx(ctx)
  const appConfig = getConfig(app)
  const { mchId, key, appId } = await appConfig.getObject('weixin_pay')
  const { site_host } = await appConfig.getObject('base')

  const payObj = new Pay({
    appId: appId,
    mchId: mchId,
    key: key, // 微信商户平台API密钥,
    pfx: fs.readFileSync(
      path.resolve(
        __dirname,
        `${process.cwd()}/app/${appName}/plugins/weixin/apiclient_cert.p12`
      )
    ), // 微信商户平台证书
  })
  const orderModel = prefix ? `${prefix}_order` : 'order'
  const out_trade_no = prefix
    ? `${appName}_${prefix}_${id}_MP-WEIXIN`
    : `${appName}_${id}_MP-WEIXIN`
  const orderDetail = await app.model[orderModel].findOne({
    where: {
      id,
    },
  })
  if (!orderDetail) throw new Error('不存在该订单')
  try {
    const { result_code, err_code_des } = await payObj.refund({
      ...rest,
      out_trade_no,
      out_refund_no: id + '',
      total_fee: Math.round((total_fee || refund_fee || price) * 100),
      refund_fee: Math.round((refund_fee || price) * 100),
      notify_url: `https://${
        site_host || 'api.kuashou.com'
      }/${appName}/weixin/refund_notify`,
    })
    if (result_code === 'SUCCESS') {
      return true
    } else {
      throw new Error(err_code_des)
    }
  } catch (e) {
    throw new Error(e)
  }
}

exports.cash = async ({ ctx, id, user_id, number }) => {
  const { app, appName } = getAppByCtx(ctx)

  const appConfig = getConfig(app)
  const { mchId, key, appId } = await appConfig.getObject('weixin_pay')

  const { is_dev } = await appConfig.getObject('base')

  const payObj = new Pay({
    appId: appId,
    mchId: mchId,
    key: key, // 微信商户平台API密钥,
    pfx: fs.readFileSync(
      path.resolve(
        __dirname,
        `${process.cwd()}/app/${appName}/plugins/weixin/apiclient_cert.p12`
      )
    ), // 微信商户平台证书
  })

  const user = await app.model.user.findOne({
    where: {
      id: user_id,
    },
    include: app.model.mp_user,
  })

  if (!user) throw new Error('找不到该用户')
  if (!user.mp_user.openid) throw new Error('找不到该用户openid')

  const amount = Math.abs(number) * 100

  const result = await payObj.transfers({
    partner_trade_no: `${id}`,
    openid: user.mp_user.openid,
    check_name: 'NO_CHECK',
    amount,
    desc: '提现',
    spbill_create_ip: '192.168.0.1',
  })

  if (result.err_code_des) throw new Error(result.err_code_des)
  return result
}
