const axios = require('axios')
const { lodash } = require('multiple-quick-koa')

const queryFn = (obj) => {
  const query = Object.keys(obj)
    .filter((item) => {
      return obj[item]
    })
    .reduce((a, b) => `${a}&${b}=${obj[b]}`, '')
  return query.replace('&', '?')
}
module.exports = class Singleton {
  constructor (key) {
    this.key = key

    /**
     * 单例模式
     */
    if (!Singleton.instance) {
      Singleton.instance = this
    }
    const previous = Singleton.instance.getConfig()
    if (!lodash.isEqual(previous, key)) {
      Singleton.instance = this
    }
    return Singleton.instance
  }

  async geo ({ address }) {
    if (!this.key) throw new Error('没有key')

    const obj = {
      key: this.key,
      address,
    }

    const url = encodeURI(
      `https://restapi.amap.com/v3/geocode/geo${queryFn(obj)}`
    )

    const data = await axios.get(url).then((res) => res.data)
    if (data.status !== '1') throw new Error(data.info)
    return data.geocodes
  }

  async regeo ({ longitude, latitude }) {
    if (!this.key) throw new Error('没有key')

    const obj = {
      key: this.key,
      location: `${longitude},${latitude}`,
    }

    const url = encodeURI(
      `https://restapi.amap.com/v3/geocode/regeo${queryFn(obj)}`
    )

    const data = await axios.get(url).then((res) => res.data)

    return data
  }

  async getDistrict ({
    keywords,
    subdistrict,
    page,
    offset,
    extensions,
    filter,
  }) {
    if (!this.key) throw new Error('没有key')

    const obj = {
      key: this.key,
      keywords,
      subdistrict,
      page,
      offset,
      extensions,
      filter,
    }

    const url = encodeURI(
      `https://restapi.amap.com/v3/config/district${queryFn(obj)}`
    )

    const data = await axios.get(url).then((res) => res.data)

    return data
  }

  getConfig () {
    return this.key
  }
}
