const { getObject } = require('../utils')

module.exports = class Config {
  constructor(app) {
    this.app = app
    this.configList = []
  }

  async init() {
    if (this.app.cache && this.app.cache.get('configList')) {
      this.configList = this.app.cache.get('configList')
      return
    }
    const configList = await this.app.model.setting.findAll({
      where: {
        type: {
          excludes: ['textarea', 'rich-text'],
        },
      },
    })
    this.app.cache && this.app.cache.set('configList', configList)
    this.configList = configList
  }

  async getObject(type) {
    if (this.app.cache && this.app.cache.get(type))
      return this.app.cache.get(type)
    await this.init()
    const result = getObject(this.configList)(type)
    if (!result) throw new Error('没有这个type')
    this.app.cache && this.app.cache.set(type, result)
    return result
  }

  update() {
    this.app.cache.set('configList', null)
  }
}
