const nodemailer = require('nodemailer');


module.exports = class Email {
  constructor(config) {
    this.config = config;
    this.transporter = null;
    this.init();
  }

  init() {
    const { email, password } = this.config;
    this.transporter = nodemailer.createTransport({
      service: 'qq',
      auth: {
        user: email,
        pass: password,
      },
    });
  }

  async send({
    to,
    subject,
    text = '',
    html,
  }) {
    const { email } = this.config;
    const mailOptions = {
      from: email, // 发送者
      to,
      subject, // 标题
      text, // 文本
      html,
    };
    return new Promise((resolve, reject) => {
      this.transporter.sendMail(mailOptions, (err, info) => {
        if (err) {
          return reject(err);
        }
        return resolve(info);
      });
    });
  }
};
