const axios = require('axios')
const util = require('util')
const { lodash } = require('multiple-quick-koa')
const toLocationUrl =
  'https://apis.map.qq.com/ws/geocoder/v1/?location=%s&key=%s'
const toLngAndLatUrl =
  'https://apis.map.qq.com/ws/geocoder/v1/?address=%s&key=%s'
module.exports = class Singleton {
  constructor(key) {
    this.key = key

    /**
     * 单例模式
     */
    if (!Singleton.instance) {
      Singleton.instance = this
    }
    const previous = Singleton.instance.getConfig()
    if (!lodash.isEqual(previous, key)) {
      Singleton.instance = this
    }
    return Singleton.instance
  }

  async direction({
    from_latitude,
    from_longitude,
    to_latitude,
    to_longitude,
    name,
  }) {
    if (!from_latitude || !from_longitude || !to_latitude || !to_longitude) {
      throw new Error('没输入起始位置')
    }
    if (!this.key) throw new Error('没有key')
    const url = `https://apis.map.qq.com/ws/direction/v1/driving/?from=${from_latitude},${from_longitude}&to=${to_latitude},${to_longitude}&output=json&callback=cb&key=${this.key}`
    const data = await axios.get(url).then((res) => res.data)

    if (data.status === 0) return data.result
    throw new Error(data.message)
  }

  async getToLoation({ longitude, latitude }) {
    if (!longitude || !latitude) throw new Error('没输入地理位置')
    if (!this.key) throw new Error('没有key')
    const data = await axios
      .get(util.format(toLocationUrl, `${latitude},${longitude}`, this.key))
      .then((res) => res.data)

    if (data.status === 0) return data.result
    throw new Error(data.message)
  }

  async getToLngAndLat(name) {
    if (!name) throw new Error('没输入地理名称')
    if (!this.key) throw new Error('没有key')
    const url = encodeURI(util.format(toLngAndLatUrl, `${name}`, this.key))
    const data = await axios.get(url).then((res) => res.data)

    if (data.status === 0) return data.result
    if (data.message === '查询无结果') {
      throw new Error(`找不到【${name}】，可能需要更详细一点（省市区）`)
    } else {
      throw new Error(data.message)
    }
  }

  getConfig() {
    return this.key
  }
}
