const util = require('util')
const axios = require('axios')
const getAccessTokenUrl = `https://api.weixin.qq.com/cgi-bin/token?grant_type=%s&appid=%s&secret=%s`
const getTicketUrl = 'https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=%s&type=jsapi'
const sha1 = require('sha1')
const LRU = require('lru-cache')
const cache = new LRU({
  max: 100,
  maxAge: 1000 * 60 * 60
})
const {
  lodash
} = require('multiple-quick-koa')

module.exports = class Singleton {
  constructor(config) {
    this.config = {
      ...config,

      noncestr: 'kuashou',
      grant_type: 'client_credential',
    }
    /**
     * 单例模式
     */
    if (!Singleton.instance) {
        Singleton.instance = this
    }
    const previous = Singleton.instance.getConfig()
    if (!lodash.isEqual(previous, config)) {
      Singleton.instance = this
    }
    return Singleton.instance
  }
  init() {
    if (!this.config.appid || !this.config.secrect) throw new Error('没有配置appid或secrect')
  }
  async getAccessToken() {
    const { appid, secrect, grant_type } = this.config
    if (cache.get('access_token')) {
      return cache.get('access_token')
    }
    const url = util.format(getAccessTokenUrl, grant_type, appid, secrect)
    const result = await axios.get(url).then(res => res.data)
    if (result.errcode) throw new Error(result.errmsg)

    cache.set('access_token', result.access_token)
    return result.access_token
  }

  async getTicket() {
    const access_token = await this.getAccessToken()
    const url = util.format(getTicketUrl, access_token)
    const result = await axios.get(url).then(res => res.data)
    if (result.errcode) throw new Error(result.errmsg)

    cache.set('ticket', result.ticket)
    return result.ticket
  }

  async sign(url) {
    const jsapi_ticket = await this.getTicket()
    const noncestr = this.config.noncestr
    const timestamp = Math.floor(Date.now() / 1000)

    const result = {
      noncestr,
      timestamp,
      url,
      jsapi_ticket,
      signature: sha1('jsapi_ticket=' + jsapi_ticket + '&noncestr=' + noncestr + '&timestamp=' + timestamp + '&url=' + url)
    }
    return result
  }

  getConfig () {
    return this.config
  }
}