const bodyParser = require('koa-bodyparser')
const compress = require('koa-compress')
const respond = require('koa-respond')
const koaBody = require('koa-body')
const xmlParser = require('koa-xml-body')
const websocket = require('koa-easy-ws')
const ratelimit = require('koa-ratelimit')
const { userAgent } = require('koa-useragent')
const staticFiles = require('koa-static')
const stringify = require('fast-json-stringify')
const path = require('path')

const rootDirectory = process.cwd()

exports.miAgent = () => userAgent

exports.miErrors = () => async (ctx, next) => {
  const { url } = ctx
  const appName = url.split('/')[1]
  const app = ctx.app[appName]
  try {
    return await next()
  } catch (e) {
    // app.event.emit('message', {
    //   type: 'error',
    //   payload: {
    //     message: e.message,
    //   },
    // });
    const errorData = {
      url: ctx.request.href,
      body: ctx.request.body,
      message: e.message,
      content: e.stack,
      header: ctx.request.header,
    }
    if (app && app.model && app.model.log) {
      app.model.log.upsert(errorData)
    }

    return ctx.ERROR(e.message || '发生错误了')
  }
}

exports.miStatic = (config) =>
  staticFiles(path.resolve(__dirname, `${rootDirectory}/${config.dir}`))

exports.miSend = () => {
  function render(json) {
    this.set('Content-Type', 'application/json')
    this.body = stringify(json)
  }
  return async (ctx, next) => {
    ctx.send = render.bind(ctx)
    await next()
  }
}

exports.miResponse = (methods) =>
  respond({
    // @ts-ignore
    methods,
  })

exports.miCors = (arr = [], ip = '192.168.0.1') => async (ctx, next) => {
  ctx.set('Access-Control-Allow-Origin', `${ctx.header.origin || '*'}`)
  ctx.set('Access-Control-Allow-Credentials', true)
  // ctx.set('Access-Control-Allow-Headers', '*');
  ctx.set(
    'Access-Control-Allow-Headers',
    ` Origin, X-Requested-With, Content-Type, client-type, Accept, ${arr
      .map((item) => `${item}-token`)
      .join(', ')}`
  )
  ctx.set('server-ip', ip)
  ctx.set('Access-Control-Allow-Methods', 'PUT, POST, GET, DELETE, OPTIONS')
  if (ctx.method === 'OPTIONS') {
    ctx.body = 200
  } else {
    await next()
  }
}

exports.miXmlBody = (config) =>
  xmlParser({
    limit: 1024,
    encoding: 'utf8', // lib will detect it from `content-type`
    xmlOptions: {
      explicitArray: false,
    },
    key: 'xmlBody', // lib will check ctx.request.xmlBody & set parsed data to it.
    onerror: (err) => {
      throw new Error(err.message)
    },
    ...config,
  })

exports.miBody = (config) =>
  koaBody({
    multipart: true,
    formLimit: '20mb',
    jsonLimit: '20mb',
    textLimit: '20mb',
    formidable: {
      maxFieldsSize: 20 * 1024 * 1024,
    },
    ...config,
  })

exports.miCompress = (config) => compress(config)

exports.miBodyParser = (config) => bodyParser(config)

exports.miWebsocket = (config) => websocket(config)

exports.miLimit = (config) => ratelimit(config)
// exports.miProxy = () => proxy('/classroom', {
//   target: 'https://ugcbsy.qq.com',
//   changeOrigin: true,
// });
